using System;
using System.Collections;

namespace NsBusinessLogic
{
	public class Order
	{
		protected NsPersistent.POrder mPersistent = null;
		static protected Delivery mCurrentDelivery = null;

		protected Order(int iCustomerId, DateTime orderDate)
		{
			mPersistent = new NsPersistent.POrder(iCustomerId, orderDate);
		}

		public Order(NsPersistent.POrder persistent)
		{
			mPersistent = persistent;
		}


		//******************** Business logic methods ******************
		
		/* Place Order is sent a XML string order as follow:
		<Order CustomerId="2" Date="25/12/2003">
		<Product Id="3" Qty="10" MinimumDeliveryQty=10/>
		<Product Id="5" Qty="15" MinimumDeliveryQty=10/>
		<Product Id="6" Qty="20" MinimumDeliveryQty=10/>
		</Order>	 
		*/
		static public Order OrderReceived(string strOrder)
		{
			System.Xml.XmlDocument doc = new System.Xml.XmlDocument();
			doc.LoadXml(strOrder);
			System.Xml.XmlNode rootNode = doc.DocumentElement;
			int iCustomerId = Convert.ToInt32(rootNode.Attributes["CustomerId"].Value);
			DateTime orderDate = DateTime.Parse(rootNode.Attributes["Date"].Value);
			System.Xml.XmlNode productNode = null;
			Order order = new Order(iCustomerId, orderDate);

			if (rootNode.HasChildNodes)
			{
				productNode = rootNode.FirstChild;

				while (productNode != null)
				{
					order.AddOrderItem(
						Convert.ToInt32(productNode.Attributes["Id"].Value), 
						Convert.ToInt32(productNode.Attributes["Qty"].Value),
						Convert.ToInt32(productNode.Attributes["MinimumDeliveryQty"].Value));

					productNode = productNode.NextSibling;
				} 
			}

			order.CalculateTotalCharge();
			return order;
		}

		protected Decimal CalculateTotalCharge()
		{
			NsPersistent.POrderItem [] aPersistents = mPersistent.GetOrderItems();
			Decimal dcTotal = 0;

			foreach (NsPersistent.POrderItem orderedProduct in aPersistents)
			{
				dcTotal += orderedProduct.Qty * orderedProduct.GetProduct().Price;
			}

			mPersistent.TotalCharge = dcTotal;
			return dcTotal;
		}

		/// <summary>
		/// Get the XML report of the delivery history of each order item.
		/// </summary>
		public string GetDeliveryReport()
		{
			OrderDeliveryReport report = new OrderDeliveryReport();
			System.Xml.XmlNode orderNode = report.AddOrderElement(
				mPersistent.OrderId, 
				mPersistent.GetCustomer().Name,
				mPersistent.GetCustomer().Address,
				mPersistent.OrderDate);

			NsPersistent.POrderItem [] aOrderItems = mPersistent.GetOrderItems();
			System.Xml.XmlNode productNode = null;

			foreach (NsPersistent.POrderItem orderItem in aOrderItems)
			{
				productNode = report.AddProductElement(orderNode, orderItem.GetProduct().ProductId, 
					orderItem.GetProduct().Description, orderItem.Qty);

				foreach (NsPersistent.PDeliveryItem deliveryItem in orderItem.GetItemDeliveries())
				{
					report.AddDeliveryElementToProductElement(productNode, deliveryItem.Qty, deliveryItem.GetDelivery().DeliveryDate);
				}
			}

			return report.GetXmlString();
		}

		/// <summary>
		/// Go through all order items and call their own CheckForDelivery methods. If a delivery 
		/// is due, generate the delivery report to be used as delivery document.
		/// </summary>
		public void CheckForDelivery()
		{
			if (mPersistent.Closed == true) 
				throw new Exception("Order closed!");

			OrderItem [] aOrderItems = GetOrderItems();
			bool fOrderClosed = true;
			
			foreach (OrderItem orderItem in aOrderItems)
			{
				if (orderItem.PendingQty > 0)
				{
					orderItem.CheckForDelivery();
					if ((orderItem.PendingQty > 0) && (fOrderClosed == true)) 
						fOrderClosed = false;
				}
			}

			if (fOrderClosed)
				mPersistent.Closed = true;

			if (mCurrentDelivery != null)
			{
				mCurrentDelivery.GenerateDeliveryReport();
				mCurrentDelivery = null;
			}
		}

		public Delivery CurrentDelivery
		{
			get 
			{ 
				if (mCurrentDelivery == null) MakeDelivery();
				return mCurrentDelivery; 
			}
		}

		static public Order [] GetAllOrders(NsPersistent.OrderType orderType)
		{
			NsPersistent.POrder [] aPersistents = NsPersistent.DataSetManager.GetAllOrders(orderType);
			Order [] aOrders = new Order[aPersistents.Length];

			for (int i = 0; i < aPersistents.Length; i++)
			{
				aOrders[i] = new Order(aPersistents[i]);
			}

			return aOrders;
		}


		//**************** Persistent properties & Navigation methods ****************

		public NsPersistent.POrder Persistent
		{
			get { return mPersistent; }
		}

		protected OrderItem AddOrderItem(int iProductId, int iQty, int iMinDeliveryQty)
		{
			return new OrderItem(mPersistent.AddOrderItem(iProductId, iQty, iMinDeliveryQty));
		}

		public OrderItem [] GetOrderItems()
		{
			NsPersistent.POrderItem [] aPersistents = mPersistent.GetOrderItems();
			OrderItem [] aOrderItems = new OrderItem[aPersistents.Length];

			for (int i = 0; i < aPersistents.Length; i++)
			{
				aOrderItems[i] = new OrderItem(aPersistents[i]);
			}

			return aOrderItems;
		}

		public Delivery MakeDelivery()
		{
			mCurrentDelivery = new Delivery(mPersistent.MakeDelivery(DateTime.Now, mPersistent.GetCustomer().Address));
			return mCurrentDelivery;
		}
	}
}
